# Teknisa Entities 

## Introdução

Este repositório contém:
- O código das entidades da base saas teknisa (usuário TEKNISA_SAAS_REFERENCE, no ambiente de produção Teknisa);
- Os arquivos ".sql" para atualização de estrutura, na pasta `scripts/reference`;
- Os objetos de banco de dados, na pasta `scripts/objects`.

As entidade são mapeadas para a biblioteca Doctrine\ORM cuja documentação pode ser encontrada [aqui][3]

## Processo de atualização de estrutura/objetos do BD TEKNISA

### **IMPORTANTE**: Toda e qualquer modificação realizada em estrutura e/ou objetos do BD **NÃO PODEM** fazer com que **versões anteriores** dos produtos deixem de funcionar.

### Demanda Mobile

1. Cliente (ou Teknisa) abre **issue** de evolução ou correção de erros (no **CP** ou **PM**);
2. Desenvolvedor analisa issue e, caso identifique necessidade de alteração de estrutura e/ou objetos do BD, cria uma nova issue de modelagem relacionada à issue de desenvolvimento:
 * A criação da issue de modelagem deve ser feita na aba **Issues relacionadas** da tela **Gerenciamento de Issue**, no **PM**;
 * A issue de modelagem criada deverá ter os campos `Produto` e `Categoria` preenchidos com **Modelagem de Dados**;
3. Automaticamente, serão criadas duas tarefas para atendimento à `issue de modelagem`: 
 * Tarefa categoria `Modelagem de dados`: será criada em nome do operador que incluiu a issue de modelagem, que será responsável pela alteração dos objetos de BD (localizados em `scripts/objects`) e/ou pela definição dos scripts de alteração da estrutura do BD;
 * Tarefa categoria `Merge Request`: será criada em nome de um operador **GCO**, que será responsável pela avaliação das modificações realizadas e homologação dos scripts modificados no aplicativo **Model Tools**;
4. Operador responsável pela issue de desenvolvimento inicia `tarefa de modelagem`, realiza as modificações de estrutura/objetos necessárias e finaliza a tarefa;
5. Operador `GCO` aloca tarefas em projeto de `subrelease de modelagem`;
6. Operador `GCO` inicia `tarefa de merge request`, avalia modificações, atualiza scripts no `Model Tools` e finaliza tarefa;
7. Operador `GCO` finaliza e homologa projeto de `subrelease de modelagem`;
8. Desenvolvedor finaliza issue original.

**OBS.:**
 * Somente após o operador GCO `finalizar o merge e o projeto de modelagem`, o desenvolvedor conseguirá finalizar a sua issue e liberá-la para teste.
 * É **importante** que a issue de modelagem seja criada relacionada à issue original (como descrito no item 2 acima), pois garantirá que a liberação seja possível somente após homologação da alteração da estrutura do BD, evitando assim, erros no ambiente do **CLIENTE**.
 * Issues de modelagem criadas sem relação com outra issue seguirão o mesmo processo, porém sem a garantia descrita no item anterior.

### Demanda Delphi

1. Cliente (ou Teknisa) abre **solicitação** de evolução ou correção de erros `Delphi` (no **CP** ou **GEP**);
2. Desenvolvedor realiza implementação necessária, seguindo o processo atual de evoluções/correções Delphi, `inclusive no que diz respeito à alterações de estrutura/objetos do BD`;
3. Desenvolvedor tramita solicitação finalizada para **GCO**;
4. GCO prepara ambiente de teste e encaminha solicitação para o setor de **Teste**;
5. Setor de teste avalia implementação e, caso não encontre problemas, encaminha solicitação para `GCO` para que seja homologada;
6. Automaticamente, `após OK do setor de teste e encaminhamento para GCO`, será criada um nova **Issue de modelagem** com duas novas tarefas (**PM**) para atendimento à `alteração de modelagem realizada na solicitação` (`isso somente acontecerá caso seja identificada, na solicitação, atividades de modelagem de dados`): 
 * Tarefa categoria `Modelagem de dados`: será criada em nome do desenvolvedor que realizou a implementação da solicitação, que será responsável pela alteração dos objetos de BD (localizados em `scripts/objects`) e/ou pela definição dos scripts de alteração da estrutura do BD;
 * Tarefa categoria `Merge Request`: será criada em nome de um operador `GCO`, que será responsável pela avaliação das modificações realizadas e homologação dos scripts modificados no aplicativo **Model Tools**;
7. Desenvolvedor responsável pela solicitação inicia `tarefa de modelagem` (**no ambiente mobile - PM**), realiza as modificações de estrutura/objetos necessárias e finaliza a tarefa;
8. Operador `GCO` aloca tarefas em projeto de `subrelease de modelagem`;
9. Operador `GCO` inicia `tarefa de merge request`, avalia modificações, atualiza scripts no `Model Tools` e finaliza tarefa;
10. Operador `GCO` finaliza e homologa projeto de `subrelease de modelagem`;
11. Operador `GCO` homologa solicitação.

**OBS.:**
 * Somente após o operador GCO `finalizar o merge e o projeto de modelagem`, a solicitação de serviço original poderá ser homologada (**isso será bloqueado pelo sistema**).
 * **Importante** ressaltar que objetos de BD (`procedures, functions, triggers, etc`), modificados para atendimento à solicitação Delphi, **deverão** também ser modificados e homologados no ambiente mobile (`scripts/objects`), durante realização do item 7 descrito acima. 

## Entidades

Para maiores detalhes leia a documentação do gerador de entidades [aqui][1]

**OBS.:**
 * Tabelas sem submodels **NÃO SÃO** importadas.
 * Para gerar uma entidade para uma tabela, a última **PRECISA** ter uma Primary Key.
 * Este projeto está configurado para **não gerar relacionamentos** entre as entidades.
   A base da teknisa possui relacionamentos(fks + pks) "complexos" demais e [não suportados pelo Doctrine][2].
 * **Não existe** suporte para campos LONG_RAW. Atualmente são gerados como BLOB,
 o que não deve funcionar corretamente.

[1]: http://192.168.122.56/zeedhi/tools/blob/master/README.md
[2]: http://stackoverflow.com/questions/19008582/doctrine2-map-entities-with-composite-foreign-keys-in-the-composite-primary-keys
[3]: http://docs.doctrine-project.org/projects/doctrine-orm/en/lat

